import karamba
import os
import re

plugin_name='Battery AIO'
icon = 'energy.png'
default_param = {
	"acdc_stateDir" : "/proc/acpi/ac_adapter/AC",
	"batt_stateDir" : "/proc/acpi/battery/BAT0",
	"batt_interval" : 5}

def init(widget):
	pass

def isOnLine(acStateDir) :
	f = open(acStateDir + "/state","r")
	ac = f.readline()
	f.close()
	acmatch = re.search("on-line", ac)
	if ( acmatch != None ) :
		return True
	else :
		return False

def getAcpiInfos(battStateDir):
	rate = 0
	remaining = 0
	capacity = 0
	state = ""
	seconds = 0

	f = open(battStateDir + "/state","r")
	acpiInfo = f.readlines()
	f.close()
	for line in acpiInfo:
		match = re.search("([a-z ]+):([ ]+)([a-zA-Z0-9-]*)",line)
		if ( match.group(1) == "present rate" ) :
			rate = float(match.group(3))
		if ( match.group(1) == "remaining capacity" ) :
			remaining = float(match.group(3))
		if ( match.group(1) == "charging state" ) :
			state = match.group(3)
	f = open(battStateDir + "/info","r")
	acpiInfo = f.readlines()
	f.close()
	for line in acpiInfo:
		match = re.search("([0-9a-z ]+):([ ]+)([a-zA-Z0-9-]*)",line)
		if (match != None):
			if ( match.group(1) == "last full capacity" ) :
				capacity = float(match.group(3))
	if ( state != "discharging" ) :
		state = "charging"
	if ( state != "discharging" and rate == 0) :
		state = "charged"
	if ( rate != 0 ) :
		if ( state != "discharging" ) :
			seconds=int(3600*(capacity-remaining)/rate)
		else :
			seconds=int(3600*(remaining/rate))

	hours = int(seconds/3600)
	seconds = seconds - 3600*hours
	minutes = int(seconds/60)
	seconds = seconds -60*minutes

	time = str(hours) + ":"
	if ( minutes < 10) :
		time = time + "0"
	time = time + str(minutes) + ":"
	if ( seconds < 10) :
		time = time + "0"
	time = time + str(seconds)

	percent = 0
	if ( capacity != 0) :
		percent = int(remaining/capacity*100)
	msg = state
	if ( state != "charged" ) :
		msg = time
	if ( state != "discharging" and percent == 100 ) :
		msg = "charged"

	return (percent, msg)

class batteryModule:
	
	def __init__(self, widget, theme, x, y):
		self.theme = theme
		self.mainImg = karamba.createImage(widget, x, y, "empty.png")
		self.bottomImg = karamba.createImage(widget, x, y + 91, theme + "acpi/battery_bottom.png")
		self.bar = karamba.createBar(widget, x, y+17, 74, 74, theme + "acpi/battery_bar.png")
		self.topImg = karamba.createImage(widget, x + 9, y, theme + "acpi/battery_top.png")
		self.maskImg = karamba.createImage(widget, x, y, theme + "acpi/battery_mask.png")
		self.loadText = karamba.createText(widget, x, y + 43, 74, 22, "")
		karamba.changeTextSize(widget, self.loadText, 20)
		self.x = x
		self.y = y
		
		karamba.hideImage(widget, self.bottomImg)
		karamba.hideImage(widget, self.topImg)
		karamba.hideImage(widget, self.maskImg)
		karamba.hideBar(widget, self.bar)
		karamba.setBarVertical(widget, self.bar, 1)
		karamba.setTextAlign(widget, self.loadText, "CENTER")
	
	def update(self, widget, isOnAc, load):
		text_x = karamba.getTextPos(widget, self.loadText)[0]
		karamba.changeText(widget, self.loadText, str(load) + "%")
		if isOnAc:
			if ( text_x != self.x + 74 ):
				karamba.moveText(widget, self.loadText, self.x + 78, self.y + 43)
			karamba.setImagePath(widget, self.mainImg, self.theme + "acpi/externalpower.png")
			karamba.hideImage(widget, self.bottomImg)
			karamba.hideImage(widget, self.topImg)
			karamba.hideImage(widget, self.maskImg)
			karamba.hideBar(widget, self.bar)
		else:
			if ( text_x != self.x + 74 ):
				karamba.moveText(widget, self.loadText, self.x, self.y + 43)
			if ( load == 0 ):
				karamba.setImagePath(widget, self.mainImg, self.theme + "acpi/battery_empty.png")
				karamba.hideImage(widget, self.bottomImg)
				karamba.hideImage(widget, self.topImg)
				karamba.hideImage(widget, self.maskImg)
				karamba.hideBar(widget, self.bar)
			elif ( load == 100):
				karamba.setImagePath(widget, self.mainImg, self.theme + "acpi/battery_full.png")
				karamba.hideImage(widget, self.bottomImg)
				karamba.hideImage(widget, self.topImg)
				karamba.hideImage(widget, self.maskImg)
				karamba.hideBar(widget, self.bar)
			else:
				top_y = int((float(100 - load) / 100.0) * 74.0) + 10 + self.y
				karamba.setImagePath(widget, self.mainImg, self.theme + "acpi/battery_bg.png")
				karamba.showImage(widget, self.bottomImg)
				karamba.showImage(widget, self.topImg)
				karamba.showImage(widget, self.maskImg)
				karamba.showBar(widget, self.bar)
				karamba.setBarValue(widget, self.bar, load)
				karamba.moveImage(widget, self.topImg, self.x + 9, top_y)
	
	def close(self, widget):
		karamba.deleteImage(widget, self.mainImg)
		karamba.deleteImage(widget, self.bottomImg)
		karamba.deleteImage(widget, self.topImg)
		karamba.deleteImage(widget, self.maskImg)
		karamba.deleteBar(widget, self.bar)
		karamba.deleteText(widget, self.loadText)

class Plugin:
	
	def __init__(self, widget, theme, color, titleColor, conf):
		self.count = conf["batt_interval"]
		self.interval = conf["batt_interval"]
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Battery")
		karamba.changeTextSize(widget, self.title, 20)
		self.line = karamba.createImage(widget, 31, 58, theme + "line.png")
		
		self.acStateDir = conf["acdc_stateDir"]
		self.battStateDir = conf["batt_stateDir"]
		self.timeText = karamba.createText(widget, 34, 114, 113, 30, "")
		karamba.setTextAlign(widget, self.timeText, "CENTER")
		self.batModule = batteryModule(widget, theme, 143, 70)
	
	def setTextShadow(self, widget, bool):
			karamba.changeTextShadow(widget, self.batModule.loadText, int(bool))
			karamba.changeTextShadow(widget, self.timeText, int(bool))
	
	def setTextColor(self, widget, color):
		karamba.changeTextColor(widget, self.batModule.loadText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.timeText, color[0], color[1], color[2])
	
	def setTextFont(self, widget, font):
		karamba.changeTextFont(widget, self.batModule.loadText, font)
		karamba.changeTextFont(widget, self.timeText, font)
	
	def update(self, widget):
		if ( self.count < self.interval ):
			self.count = self.count + 1
		else:
			self.count = 1
			if (os.path.exists(self.acStateDir + "/state") and os.path.exists(self.battStateDir + "/state") and os.path.exists(self.battStateDir + "/info")):
				infos = getAcpiInfos(self.battStateDir)
				status = isOnLine(self.acStateDir)
				self.batModule.update(widget, status, infos[0])
				text = infos[1]
				if status:
					if ( text == "charged" ):
						karamba.changeText(widget, self.timeText, "charged")
					else:
						karamba.changeText(widget, self.timeText, text + "\nuntil charged")
				else:
					karamba.changeText(widget, self.timeText, text + "\nremaining")
			else:
				karamba.changeText(widget, self.timeText, "Configure\nyour paths.")
	
	def commandOutput(self, widget, pid, output):
		pass
	
	def meterClicked(self, widget, meter, button):
		pass
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		karamba.deleteText(widget, self.timeText)
		self.batModule.close(widget)