import karamba

plugin_name = "Mail AIO"
icon = 'email.png'
default_param = {
		"mail_list" : [],
		"mail_interval" : 1200}

def init(widget):
	pass

class MailModule:
	
	def __init__(self, widget, theme, color, x, y, name, server, username, password, conf):
		self.serv = server
		self.user = str(username)
		self.passw = str(password)
		self.conf = conf
		self.name = karamba.createText(widget, x, y + 8, 124, 18, name)
		self.imgMail = karamba.createImage(widget, x + 124, y, theme + "mail.png")
		self.num = karamba.createText(widget, x + 124, y + 8, 32, 18, "")
		self.text = karamba.createText(widget, x + 156, y + 8, 124, 18, "new message(s)")
		karamba.setTextAlign(widget, self.name, "CENTER")
		karamba.setTextAlign(widget, self.num, "CENTER")
		karamba.setTextAlign(widget, self.text, "CENTER")
		karamba.changeTextSize(widget, self.name, 14)
		karamba.changeTextSize(widget, self.num, 14)
		karamba.changeTextSize(widget, self.text, 14)
		karamba.attachClickArea(widget, self.imgMail)
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.name, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.num, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.text, color[0], color[1], color[2])
	
	def setTextFont(self, widget, font):
		karamba.changeTextFont(widget, self.name, font)
		karamba.changeTextFont(widget, self.num, font)
		karamba.changeTextFont(widget, self.text, font)
	
	def shadow(self, widget, bool):
		karamba.changeTextShadow(widget, self.name, int(bool))
		karamba.changeTextShadow(widget, self.num, int(bool))
		karamba.changeTextShadow(widget, self.text, int(bool))
	
	def update(self, widget):
		path = self.conf["scripts_dir"]
		if (self.serv == "gmail"):
			command = ["python", path + "getGMail.py", self.user, self.passw]
		elif (self.serv.find("imap") != -1):
			command = ["python", path + "getNumMail_IMAP.py", self.serv, self.user, self.passw]
		else:
			command = ["python", path + "getNumMail_POP.py", self.serv, self.user, self.passw]
		self.pid = karamba.executeInteractive(widget, command)
	
	def close(self, widget):
		karamba.deleteImage(widget, self.imgMail)
		karamba.deleteText(widget, self.name)
		karamba.deleteText(widget, self.num)
		karamba.deleteText(widget, self.text)

class Plugin:
	
	def __init__(self, widget, theme, color, titleColor, conf):
		self.interval = conf["mail_interval"]
		self.count = conf["mail_interval"]
		self.module_list = []
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Mail")
		karamba.changeTextSize(widget, self.title, 20)
		self.line = karamba.createImage(widget, 31, 58, theme + "line.png")
		
		number = len(conf["mail_list"]) / 4
		space = (129 - 32*number) / (number + 1)
		incr = space + 32
		y = 56 + space
		for i in range(number):
			self.module_list.append(MailModule(widget, theme, color, 40, y, conf["mail_list"][ i * 4 ], conf["mail_list"][ i * 4 + 1 ], conf["mail_list"][ i * 4 + 2 ], conf["mail_list"][ i * 4 + 3 ], conf))
			y = y + incr
	
	def setTextColor(self, widget, color):
		for module in self.module_list:
			module.updateColor(widget, color)
	
	def setTextFont(self, widget, font):
		for module in self.module_list:
			module.setTextFont(widget, font)
	
	def setTextShadow(self, widget, bool):
		for module in self.module_list:
			module.shadow(widget, bool)
	
	def update(self, widget):
		if ( self.count < self.interval ):
			self.count = self.count + 1
		else:
			self.count = 1
			for module in self.module_list:
				module.update(widget)
	
	def commandOutput(self, widget, pid, output):
		for module in self.module_list:
			if ( pid == module.pid ):
				num = output.replace('\n','')
				karamba.changeText(widget, module.num, num)
				text = karamba.getTextValue(widget, module.name)
				karamba.changeText(widget, module.text, "new message(s)")
				print "** " + text + " updated **"
				karamba.redrawWidget(widget)
		karamba.redrawWidget(widget)
	
	def meterClicked(self, widget, meter, button):
		if ( button == 1 ):
			for module in self.module_list:
				if ( meter == module.imgMail ):
					karamba.changeText(widget, module.text, "Updating")
					module.update(widget)
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		for module in self.module_list:
			module.close(widget)
