import karamba
import re
import os

plugin_name = "Network AIO"
icon = 'network.png'
default_param = {
		"net_list" : ['eth0', 512, 64],
		"net_interval" : 1200}

def init(widget):
	pass

class NetworkModule:
	
	def __init__(self, widget, theme, color, x, y, iface, max_in, max_out, conf):
		self.theme = theme
		self.page = 1
		self.iface = iface
		self.conf = conf
		iface_text = iface[0].upper() + iface[1:]
		self.ifaceText = karamba.createText(widget, x, y + 6, 100, 20, iface_text)
		karamba.changeTextSize(widget, self.ifaceText, 18)
	
		self.wireless = self.getWireless(iface)
		if self.wireless:
			self.signal = karamba.createText(widget, x+78, y + 14, 30, 14, "")
			karamba.setTextAlign(widget, self.signal, "RIGHT")
			karamba.changeTextSize(widget, self.signal, 10)
			self.icon = karamba.createImage(widget, x + 108, y, theme + "network/wireless.png")
			path = self.conf["scripts_dir"]	
			karamba.changeTextColor(widget, self.signal, color[0], color[1], color[2])
			karamba.setTextSensor(widget, self.signal, 'sensor=program interval=1000 program="python ' + path + 'getStrength.py ' + iface + '"')
		else:
			self.icon = karamba.createImage(widget, x + 108, y, theme + "network/network.png")
		karamba.attachClickArea(widget, self.icon)
		self.statusImg = karamba.createImage(widget, x + 131, y + 23, "empty.png")
		self.graphIn = karamba.createGraph(widget, x + 2, y + 34, 136, 26, 0)
		self.graphOut = karamba.createGraph(widget, x + 2, y + 34, 136, 26, 0)
		self.grid = karamba.createImage(widget, x, y + 32, theme + "bar_grid.png")
		karamba.setGraphColor(widget, self.graphIn, 0, 230, 0)
		karamba.setGraphColor(widget, self.graphOut, 230, 0, 0)
		karamba.setGraphMinMax(widget, self.graphIn, 0, max_in)
		karamba.setGraphMinMax(widget, self.graphOut, 0, max_out)
		karamba.setGraphSensor(widget, self.graphIn, 'sensor=network interval=1000 format="%in" device="' + iface + '"')
		karamba.setGraphSensor(widget, self.graphOut, 'sensor=network interval=1000 format="%out" device="' + iface + '"')
		
		self.status = karamba.createText(widget, x, y + 64, 140, 12, "")
		karamba.setTextAlign(widget, self.status, "CENTER")
		self.ipText = karamba.createText(widget, x, y + 78, 140, 12, "")
		karamba.setTextAlign(widget, self.ipText, "LEFT")
		self.trafficIn = karamba.createText(widget, x, y + 92, 70, 30, "")
		karamba.setTextSensor(widget, self.trafficIn, 'sensor=network interval=1000 decimals=1 format="Download :\n%in kB/s" device="' + iface + '"')
		self.trafficOut = karamba.createText(widget, x + 71, y + 92, 70, 30, "")
		karamba.setTextAlign(widget, self.trafficOut, "LEFT")
		karamba.setTextSensor(widget, self.trafficOut, 'sensor=network interval=1000 decimals=1 format="Upload :\n%out kB/s" device="' + iface + '"')
		for meter in [self.ipText, self.trafficIn, self.trafficOut]:
			karamba.attachClickArea(widget, meter)
		
		karamba.changeTextColor(widget, self.ifaceText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.ipText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficIn, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficOut, color[0], color[1], color[2])

	def getWireless(self, whichIface):
		found = False
		if (os.path.exists("/proc/net/wireless")):
			f = open("/proc/net/wireless","r")
			wirelessInfo = f.readlines()
			f.close()
			lineNr = 0
			for line in wirelessInfo:
				if lineNr >= 2:
					ifaceValues = line.split()
					iface = ifaceValues[0][:-1]
					if ( whichIface == iface ) :
						found = True
				lineNr = lineNr + 1
		return found
	

	def toggleConnected(self, widget, ip):
		karamba.setImagePath(widget, self.statusImg, self.theme + "network/on.png")
		karamba.changeText(widget, self.status, "Connected")
		karamba.changeTextColor(widget, self.status, 0, 230, 0)
		if (self.page == 1):
			karamba.changeText(widget, self.ipText, "Ip : " + ip)
		elif (self.page == 2):
			karamba.changeText(widget, self.ipText, "External Ip : " + ip)
		karamba.showText(widget, self.ipText)
		karamba.showText(widget, self.trafficIn)
		karamba.showText(widget, self.trafficOut)
		
	def toggleConnecting(self, widget):
		karamba.setImagePath(widget, self.statusImg, "")
		karamba.changeText(widget, self.status, "Connecting...")
		karamba.changeTextColor(widget, self.status, 255, 150, 0)
	
	def toggleDisconnected(self, widget):
		karamba.setImagePath(widget, self.statusImg, self.theme + "network/off.png")
		karamba.changeText(widget, self.status, "Disconnected")
		karamba.changeTextColor(widget, self.status, 230, 0, 0)
		karamba.hideText(widget, self.ipText)
		karamba.hideText(widget, self.trafficIn)
		karamba.hideText(widget, self.trafficOut)
	
	def shadow(self, widget, bool):
		karamba.changeTextShadow(widget, self.ifaceText, int(bool))
		if self.wireless:
			karamba.changeTextShadow(widget, self.signal, int(bool))
		karamba.changeTextShadow(widget, self.ipText, int(bool))
		karamba.changeTextShadow(widget, self.trafficIn, int(bool))
		karamba.changeTextShadow(widget, self.trafficOut, int(bool))
	
	def updateColor(self, widget, color):
		karamba.changeTextColor(widget, self.ifaceText, color[0], color[1], color[2])
		if self.wireless:
			karamba.changeTextColor(widget, self.signal, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.ipText, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficIn, color[0], color[1], color[2])
		karamba.changeTextColor(widget, self.trafficOut, color[0], color[1], color[2])
	
	def setTextFont(self, widget, font):
		karamba.changeTextFont(widget, self.ifaceText, font)
		if self.wireless:
			karamba.changeTextFont(widget, self.signal, font)
		karamba.changeTextFont(widget, self.ipText, font)
		karamba.changeTextFont(widget, self.trafficIn, font)
		karamba.changeTextFont(widget, self.trafficOut, font)
	
	def update(self, widget):
		ip = karamba.getIp(widget, self.iface)
		state = karamba.getTextValue(widget, self.status)
		if ( ip == "Disconnected" and state != "Connecting..." ):
			self.toggleDisconnected(widget)
		if ( ip != "Disconnected" ):
			if (self.page == 2):
				path = self.conf["scripts_dir"]
				f = os.popen("python " + path + "getExternalIP.py")
				ip = f.read()[0:-1]
				f.close()
			self.toggleConnected(widget, ip)
	
	def close(self, widget):
		karamba.deleteText(widget, self.ifaceText)
		if self.wireless:
			karamba.deleteText(widget, self.signal)
		karamba.deleteText(widget, self.ipText)
		karamba.deleteText(widget, self.trafficIn)
		karamba.deleteText(widget, self.trafficOut)
		karamba.deleteText(widget, self.status)
		karamba.deleteImage(widget, self.icon)
		karamba.deleteImage(widget, self.statusImg)
		karamba.deleteImage(widget, self.grid)
		karamba.deleteGraph(widget, self.graphIn)
		karamba.deleteGraph(widget, self.graphOut)


class Plugin:
	
	def __init__(self, widget, theme, color, titleColor, conf):
		self.title = karamba.createText(widget, 36, 32, 200, 24, "Network")
		karamba.changeTextSize(widget, self.title, 20)
		karamba.changeTextColor(widget, self.title, titleColor[0], titleColor[1], titleColor[2])
		self.line = karamba.createImage(widget, 31, 58, theme + "line.png")
		
		self.conf = conf
		self.interval = conf["net_interval"]
		self.count = conf["net_interval"]
		iface_list = conf["net_list"]
		self.length = len(iface_list)
		if ( self.length == 3 ):
			self.module1 = NetworkModule(widget, theme, color, 110, 60, iface_list[0], iface_list[1], iface_list[2], conf)
		elif ( self.length == 6 ):
			self.module1 = NetworkModule(widget, theme, color, 36, 60, iface_list[0], iface_list[1], iface_list[2], conf)
			self.module2 = NetworkModule(widget, theme, color, 184, 60, iface_list[3], iface_list[4], iface_list[5], conf)
	
	def setTextColor(self, widget, color):
		self.module1.updateColor(widget, color)
		if ( self.length == 6 ):
			self.module2.updateColor(widget, color)
	
	def setTextFont(self, widget, font):
		self.module1.setTextFont(widget, font)
		if ( self.length == 6 ):
			self.module2.setTextFont(widget, font)
	
	def setTextShadow(self, widget, bool):
		self.module1.shadow(widget, bool)
		if ( self.length == 6 ):
			self.module2.shadow(widget, bool)
	
	def update(self, widget):
		status = karamba.getTextValue(widget, self.module1.status)
		if ( status == "Connecting..." or self.count >= self.interval ):
			self.module1.update(widget)
			if ( self.length == 3 ):
				self.count = 1
		if ( self.length == 6 ):
			status2 = karamba.getTextValue(widget, self.module2.status)
			if ( status2 == "Connecting..." or self.count >= self.interval ):
				self.module2.update(widget)
				self.count = 1
		self.count = self.count + 1
	
	def commandOutput(self, widget, pid, output):
		pass
	
	def meterClicked(self, widget, meter, button):
		if ( meter == self.module1.icon and button == 1 ):
			status = karamba.getTextValue(widget, self.module1.status)
			if ( status == "Disconnected" ):
				karamba.execute("sudo /sbin/ifup " + self.module1.iface)
				self.module1.toggleConnecting(widget)
			elif ( status == "Connected" ):
				karamba.execute("sudo /sbin/ifdown " + self.module1.iface)
				self.module1.toggleDisconnected(widget)
		if ( self.length == 6 ):
			if ( meter == self.module2.icon and button == 1 ):
				status = karamba.getTextValue(widget, self.module2.status)
				if ( status == "Disconnected" ):
					karamba.execute("sudo /sbin/ifup " + self.module2.iface)
					self.module2.toggleConnecting(widget)
				elif ( status == "Connected" ):
					karamba.execute("sudo /sbin/ifdown " + self.module2.iface)
					self.module2.toggleDisconnected(widget)
		if ( button == 1 ):
			mod_list = [self.module1]
			if (self.length == 6 ):
				mod_list.append(self.module2)
			for module in mod_list:
				if ( meter in [module.ipText, module.trafficIn, module.trafficOut] ):
					if (module.page == 1):
						module.page = 2
						path = self.conf["scripts_dir"]
						karamba.setTextSensor(widget, module.trafficIn, 'sensor=program interval=5000 program="python ' + path + 'getIfaceTotal.py -d ' + module.iface + '"')
						karamba.setTextSensor(widget, module.trafficOut, 'sensor=program interval=5000 program="python ' + path + 'getIfaceTotal.py -u ' + module.iface + '"')
					elif (module.page == 2):
						module.page = 1
						karamba.setTextSensor(widget, module.trafficIn, 'sensor=network interval=1000 decimals=1 format="Download :\n%in kB/s" device="' + module.iface + '"')
						karamba.setTextSensor(widget, module.trafficOut, 'sensor=network interval=1000 decimals=1 format="Upload :\n%out kB/s" device="' + module.iface + '"')
					module.update(widget)
					karamba.redrawWidget(widget)
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		self.module1.close(widget)
		if ( self.length == 6 ):
			self.module2.close(widget)
