import karamba

plugin_name = "Partitions AIO"
icon = 'hdd.png'
default_param = {"partition_list" : ['Root', '/', 'Home', '/home'],
                 "partition_showFreeSpace" : False}

def init(widget):
	pass

class HddModule:
	
	def __init__(self, widget, theme, color, x, y, name, mountpoint, free_space):
		self.name = karamba.createText(widget, x, y, 100, 12, name)
		self.percent = karamba.createText(widget, x + 100, y, 40, 12, "")
		self.used_total = karamba.createText(widget, x, y + 21, 140, 12, "")
		
		karamba.setTextAlign(widget, self.percent, "RIGHT")
		karamba.setTextAlign(widget, self.used_total, "CENTER")
		karamba.setTextSensor(widget, self.percent, 'sensor=disk mountpoint=' + mountpoint + ' format="%up%" interval=10000')
		if free_space:
			karamba.setTextSensor(widget, self.used_total, 'sensor=disk mountpoint=' + mountpoint + ' format="%f of %t MB" interval=10500')
		else:
			karamba.setTextSensor(widget, self.used_total, 'sensor=disk mountpoint=' + mountpoint + ' format="%u of %t MB" interval=10500')
		
		self.bar = karamba.createBar(widget, x + 2, y + 14, 136, 6, theme + "bar_140.png")
		self.border = karamba.createImage(widget, x, y + 12, theme + "border_140.png")
		karamba.setBarSensor(widget, self.bar, 'sensor=disk mountpoint=' + mountpoint + ' interval=11000')
	
	def close(self, widget):
		karamba.deleteText(widget, self.name)
		karamba.deleteText(widget, self.percent)
		karamba.deleteText(widget, self.used_total)
		karamba.deleteImage(widget, self.border)
		karamba.deleteBar(widget, self.bar)


class Plugin:
	
	def __init__(self, widget, theme, color, titleColor, conf):
		self.loaded = False
		self.pid = None
		self.list_modules = []
		self.title = karamba.createText(widget, 36, 34, 150, 20, "Partitions")
		karamba.changeTextSize(widget, self.title, 20)
		self.line = karamba.createImage(widget, 31, 58, theme + "line.png")
		
		list_mount = conf["partition_list"]
		num = len(list_mount) / 2
		if ( num <= 3 ):
			x = 110
			num_line = num
		elif ( num <= 8 ) :
			num_line = num / 2 + num % 2
			x = 37
		y = (129 - num_line * 28) / (num_line + 1)
		y_incr = y + 28
		y = y + 56
		for i in  range(num):
			self.list_modules.append(HddModule(widget, theme, color, x, y, list_mount[2*i], list_mount[2*i+1], conf["partition_showFreeSpace"]))
			if ( num <= 3 ):
				y = y + y_incr
			elif ( num <= 8 ) :
				if ( x == 183 ):
					x = 37
					y = y + y_incr
				elif ( x == 37 ):
					x = 183
	
	def setTextColor(self, widget, color):
		for module in self.list_modules:
			karamba.changeTextColor(widget, module.name, color[0], color[1], color[2])
			karamba.changeTextColor(widget, module.percent, color[0], color[1], color[2])
			karamba.changeTextColor(widget, module.used_total, color[0], color[1], color[2])
	
	def setTextFont(self, widget, font):
		for module in self.list_modules:
			karamba.changeTextFont(widget, module.name, font)
			karamba.changeTextFont(widget, module.percent, font)
			karamba.changeTextFont(widget, module.used_total, font)
	
	def setTextShadow(self, widget, bool):
		for module in self.list_modules:
			karamba.changeTextShadow(widget, module.name, int(bool))
			karamba.changeTextShadow(widget, module.percent, int(bool))
			karamba.changeTextShadow(widget, module.used_total, int(bool))
	
	def update(self, widget):
		pass
	
	def commandOutput(self, widget, pid, output):
		pass
	
	def meterClicked(self, widget, meter, button):
		pass
	
	def close(self, widget):
		karamba.deleteText(widget, self.title)
		karamba.deleteImage(widget, self.line)
		for module in self.list_modules:
			module.close(widget)
