#!/usr/bin/python
#Usage : python config_dialog.py [CONF_PYDICT]

import sys
import re
import os
import zipfile
from time import time

try:
	from qt import *
except:
	os.popen("kdialog --sorry 'You need PyQt to use Aero AIO configuration utility.'")
	ok = False
else:
	ok = True

import config_dialog_ui, plugins_management_ui, appearance_ui, auto_switching_ui, fdialog_ui


arg = compile("conf = " + sys.argv[1], '<string>', 'exec')
exec arg
sys.path.insert(0, conf["home"] + ".aero_aio/scripts/")



def __globalimport__(module_dict):
	for module in module_dict:
		f = compile("global " + module + "\n" + module + " = __import__('" + module + "', globals(), locals(), [])", '<string>', 'exec')
		exec f

class SkzFile:
	
	def __init__(self, conf):
		self.file = zipfile.ZipFile(conf["theme_path"], 'r')
		self.file_list = self.file.namelist()
		self.conf_path = conf["home"] + ".superkaramba/aero_aio/conf"
	
	def guiDict(self):
		f = os.popen("ls -1 " + conf["home"] + ".superkaramba/aero_aio")
		a = f.readlines()
		f.close()
		pdict = {}
		for filename in a:
			match = re.search("([\S]+_gui).py\n", filename)
			if ( match != None ):
				pdict[match.group(1)] = conf["home"] + ".superkaramba/aero_aio/" + match.group(1) + ".py"
		return pdict
	
	def themeDict(self):
		self.theme_dict = {}
		for i in self.file_list:
			if (i.find('themes/') != -1 and i.find('/theme',7) != -1 and i.find('/theme~',7) == -1):
				theme = i[7:-6]
				string = self.file.read('themes/' + theme + '/theme')
				endline = string.find('\n')
				string = string[:endline]
				f = compile(string, '<string>', 'exec')
				exec f
				self.theme_dict[name] = 'themes/' + theme + "/"
		return self.theme_dict
	
	def iconDict(self):
		self.icon_dict = {}
		for i in self.file_list:
			if (i.find('icons/') != -1 and i.find('/pc.png',6) != -1):
				name = i[6:-7]
				self.icon_dict[name] = 'icons/' + name + "/"
		return self.icon_dict
	
	def themePreviewQPixmap(self, name):
		path = self.theme_dict[name]
		img_data = self.file.read(path + "preview.png")
		theme_img_pixmap = QPixmap()
		theme_img_pixmap.loadFromData(img_data,"PNG")
		return theme_img_pixmap
	
	def themeFile(self, name):
		path = self.theme_dict[name]
		data = self.file.read(path + "theme")
		return data
	
	def iconQPixmap(self):
		image0_data = self.file.read("scripts/icon.png")
		img = QPixmap()
		img.loadFromData(image0_data,"PNG")
		return img
	
	def logoQPixmap(self):
		image0_data = self.file.read("scripts/logo.png")
		img = QPixmap()
		img.loadFromData(image0_data,"PNG")
		return img
	
	def close(self):
		self.file.close()

class Dir:
	
	def __init__(self, conf):
		self.conf_path = conf["theme_path"] + "conf"
	
	def guiDict(self):
		f = os.popen("ls -1 " + conf["theme_path"] + "scripts")
		a = f.readlines()
		f.close()
		pdict = {}
		for filename in a:
			match = re.search("([\S]+_gui).py\n", filename)
			if ( match != None ):
				pdict[match.group(1)] = match.group(1)
		return pdict
	
	def themeDict(self):
		self.theme_dict = {}
		f = os.popen("ls -1 " + conf["theme_path"] + "themes")
		theme_list = f.read()
		f.close()
		theme_list = theme_list.split('\n')
		if ( theme_list[-1] == ""):
			theme_list =  theme_list[:-1]
		for theme in theme_list:
			f = open(conf["theme_path"] + 'themes/' + theme + '/theme')
			string = f.readline()
			f.close()
			f = compile(string, '<string>', 'exec')
			exec f
			self.theme_dict[name] = 'themes/' + theme + "/"
		return self.theme_dict
	
	def iconDict(self):
		self.icon_dict = {}
		f = os.popen("ls -1 " + conf["theme_path"] + "icons")
		icon_list = f.read()
		f.close()
		icon_list = icon_list.split('\n')
		if ( icon_list[-1] == ""):
			icon_list =  icon_list[:-1]
		for icon in icon_list:
			self.icon_dict[icon] = 'icons/' + icon + "/"
		return self.icon_dict
	
	def themePreviewQPixmap(self, name):
		img = QImage(conf["theme_path"] + self.theme_dict[name] + 'preview.png')
		theme_img_pixmap = QPixmap(img)
		return theme_img_pixmap
	
	def themeFile(self, name):
		f = open(conf["theme_path"] + self.theme_dict[name] + 'theme')
		data = f.read()
		f.close()
		return data
	
	def iconQPixmap(self):
		img = QImage(conf["theme_path"] + 'scripts/icon.png')
		img_pixmap = QPixmap(img)
		return img_pixmap
	
	def logoQPixmap(self):
		img = QImage(conf["theme_path"] + 'scripts/logo.png')
		img_pixmap = QPixmap(img)
		return img_pixmap
	
	def close(self):
		pass

class DataIface:
	
	def __init__(self, conf):
		if conf["skz"]:
			self.themeData = SkzFile(conf)
		else:
			self.themeData = Dir(conf)
		self.conf_path = self.themeData.conf_path
		
		self.extGui_list = []
		self.gui_dict = self.themeData.guiDict()
		f = os.popen("ls -1 " + conf["home"] + ".aero_aio/scripts")
		a = f.readlines()
		f.close()
		for filename in a:
			match = re.search("([\S]+_gui).py\n", filename)
			if ( match != None ):
				self.gui_dict[match.group(1)] = conf["home"] + ".aero_aio/scripts/" + match.group(1) + ".py"
				self.extGui_list.append(match.group(1))
		
		del_list = []
		for gui in self.gui_dict:
			plugin = gui[:-3] + "aio"
			if not(plugin in conf["plugin_dict"]):
				del_list.append(gui)
		for gui in del_list:
			del self.gui_dict[gui]
		
		__globalimport__(self.gui_dict)
		
		self.theme_dict = self.themeData.themeDict()
		self.extTheme_list = []
		self.outTheme_list = []
		f = os.popen("ls -1 " + conf["home"] + ".aero_aio/themes")
		theme_list = f.read()
		f.close()
		theme_list = theme_list.split('\n')
		if ( theme_list[-1] == ""):
			theme_list =  theme_list[:-1]
		for theme in theme_list:
			f = open(conf["home"] + ".aero_aio/themes/" + theme + '/theme')
			string = f.readline()
			f.close()
			f = compile(string, '<string>', 'exec')
			exec f
			self.theme_dict[name] = conf["home"] + ".aero_aio/themes/" + theme + '/'
			self.extTheme_list.append(name)
		for path in conf["ext_themes"]:
			f = open(path + 'theme')
			string = f.readline()
			f.close()
			f = compile(string, '<string>', 'exec')
			exec f
			self.theme_dict[name] = path
			self.extTheme_list.append(name)
			self.outTheme_list.append(name)
		self.theme_qlist = QStringList()
		for name in self.theme_dict:
			self.theme_qlist.append(name)
		
		self.icon_dict = {}
		self.extIcon_list = []
		self.outIcon_list = []
		f = os.popen("ls -1 " + conf["home"] + ".aero_aio/icons")
		icon_list = f.read()
		f.close()
		icon_list = icon_list.split('\n')
		if ( icon_list[-1] == ""):
			icon_list =  icon_list[:-1]
		for icon in icon_list:
			self.icon_dict[icon] = conf["home"] + ".aero_aio/icons/" + icon + "/"
			self.extIcon_list.append(icon)
		dict = self.themeData.iconDict()
		for icon in dict:
			self.icon_dict[icon] = dict[icon]
		for path in conf["ext_icons"]:
			match = re.search("/([^/]+)/?$", path)
			if (match != None):
				self.icon_dict[match.group(1)] = path
				self.extIcon_list.append(match.group(1))
				self.outIcon_list.append(match.group(1))
		self.icon_qlist = QStringList()
		for name in self.icon_dict:
			self.icon_qlist.append(name)
	
	def addTheme(self, path):
		try:
			f = open(path + 'theme')
			string = f.readline()
			f.close()
			f = compile(string, '<string>', 'exec')
			exec f
			self.theme_dict[name] = path
			self.extTheme_list.append(name)
			if not(name in self.outTheme_list):
				self.outTheme_list.append(name)
			if not(name in self.theme_qlist):
				self.theme_qlist.append(name)
		except:
			return False
		else:
			return name
	
	def addIcons(self, path):
		match = re.search("/([^/]+)/?$", path)
		if (match != None):
			name = match.group(1)
			self.icon_dict[name] = path
			self.extIcon_list.append(name)
			if not(name in self.outIcon_list):
				self.outIcon_list.append(name)
			if not(name in self.icon_qlist):
				self.icon_qlist.append(name)
			return name
		else:
			return False
	
	def iconQPixmap(self):
		return self.themeData.iconQPixmap()
	
	def logoQPixmap(self):
		return self.themeData.logoQPixmap()
	
	def themeName(self, path):
		lfound = None
		for name in self.theme_dict:
			if (self.theme_dict[name] == path):
				lfound = name
		return lfound
	
	def iconName(self, path):
		lfound = None
		for name in self.icon_dict:
			if (self.icon_dict[name] == path):
				lfound = name
		return lfound
		
	
	def themePreviewQPixmap(self, name):
		if (name in self.extTheme_list):
			img = QImage(self.theme_dict[name] + 'preview.png')
			theme_img_pixmap = QPixmap(img)
		else:
			theme_img_pixmap = self.themeData.themePreviewQPixmap(name)
		return theme_img_pixmap
	
	def themeDefault(self, name):
		if (name in self.extTheme_list):
			f = open(self.theme_dict[name] + 'theme')
			data = f.read()
			f.close()
		else:
			data = self.themeData.themeFile(name)
		match = re.search("color=(\[.*\])\ntitle_color=(\[.*\])\nicon_set=(.*)", data)
		color = eval(match.group(1))
		color = QColor(color[0], color[1], color[2])
		titleColor = eval(match.group(2))
		titleColor = QColor(titleColor[0], titleColor[1], titleColor[2])
		icon = match.group(3)
		return {"color" : color,
			"titleColor" : titleColor,
			"icon" : icon}
	
	def close(self):
		self.themeData.close()

def kdeIconsPath():
	f = open(conf["home"] + ".kde/share/config/kdeglobals")
	file = f.read()
	f.close()
	i1 = file.find("[Icons]\nTheme=") + 14
	i2 = file.find("\n", i1)
	name = file[i1:i2]
	
	f = os.popen("kde-config --path icon")
	path_list = f.read()
	f.close()
	path_list = path_list.replace('\n','').split(':')
	
	icon_path = ""
	for path in path_list:
		if (os.path.exists(path)):
			f = os.popen("ls " + path)
			name_list = f.read()
			f.close()
			if ( name_list.find(name + "\n") != -1 ):
				icon_path = path + name + "/"
	
	if (icon_path == ""):
		for path in path_list:
			if (os.path.exists(path)):
				f = os.popen("ls " + path)
				name_list = f.read()
				f.close()
				if ( name_list.find("default\n") != -1 ):
					icon_path = path + "default/"
	
	return icon_path

kde_iconPath = kdeIconsPath()
conf["kde_iconPath"] = kde_iconPath

dataBase = DataIface(conf)

class AIOFont:
	
	def __init__(self, family, color, shadow):
		self.family = family
		self.color = color
		self.shadow = shadow

class FDialog(fdialog_ui.FDialog):
	
	def __init__(self, *args):
		apply(fdialog_ui.FDialog.__init__, (self,) + args)
		self.connect(self.listBox, SIGNAL("highlighted ( const QString & )"), self.listBoxItemSelected)
		self.connect(self.colorButton, SIGNAL("released()"), self.button_colorClicked)
		self.color = QColor()
		self.pixmap_color = QPixmap(140,14,-1)
	
	def listBoxItemSelected(self, string):
		self.previewLabel.setText(string)
		self.previewLabel.setFont(QFont(string, 10))
	
	def button_colorClicked(self):
		self.color = QColorDialog.getColor(self.color, self, "Choose text color")
		self.pixmap_color.fill(self.color)
		self.colorButton.setPixmap(self.pixmap_color)
		self.previewLabel.setPaletteForegroundColor(self.color)

class AIOFontDialog:
	
	def __init__(self):
		pass
	
	def getFont(self, parent, family, color, shadow):
		dialog = FDialog(parent)
		data = QFontDatabase()
		font_list = data.families()
		index = font_list.findIndex(family)
		dialog.listBox.insertStringList(font_list)
		item = dialog.listBox.item(index)
		dialog.listBox.setCurrentItem(index)
		dialog.color = color
		dialog.previewLabel.setPaletteForegroundColor(color)
		dialog.pixmap_color.fill(color)
		dialog.colorButton.setPixmap(dialog.pixmap_color)
		dialog.checkBox.setChecked(shadow)
		dialog.exec_loop()
		if (dialog.result() == dialog.Accepted):
			return AIOFont(dialog.listBox.currentText(), dialog.color, dialog.checkBox.isChecked())
		else:
			return None

class ConfListViewItem(QListViewItem):
	
	def __init__(self, parent, label, stackid):
		apply(QListViewItem.__init__, (self, parent, label))
		self.stackid = stackid

class PluginsPage(plugins_management_ui.Plugin_Mgt):
	
	def __init__(self, *args):
		apply(plugins_management_ui.Plugin_Mgt.__init__, (self,) + args)
		
		imgClear = QImage(kde_iconPath + "22x22/actions/editdelete.png")
		imgClear_pixmap = QPixmap(imgClear)
		self.buttonClear.setPixmap(imgClear_pixmap)
		
		imgRight = QImage(kde_iconPath + "22x22/actions/forward.png")
		imgRight_pixmap = QPixmap(imgRight)
		self.buttonRight.setPixmap(imgRight_pixmap)
		
		imgLeft = QImage(kde_iconPath + "22x22/actions/back.png")
		imgLeft_pixmap = QPixmap(imgLeft)
		self.buttonLeft.setPixmap(imgLeft_pixmap)
		
		imgUp = QImage(kde_iconPath + "22x22/actions/up.png")
		imgUp_pixmap = QPixmap(imgUp)
		self.buttonUp.setPixmap(imgUp_pixmap)
		
		imgDown = QImage(kde_iconPath + "22x22/actions/down.png")
		imgDown_pixmap = QPixmap(imgDown)
		self.buttonDown.setPixmap(imgDown_pixmap)
		
		self.connect(self.buttonUp, SIGNAL("released()"), self.buttonUpClicked)
		self.connect(self.buttonDown, SIGNAL("released()"), self.buttonDownClicked)
	
	def selectedItems(self, listBox):
		number = listBox.count()
		item_list = []
		for i in range(number):
			if listBox.isSelected(i):
				item_list.append(listBox.item(i))
		return item_list
	
	def buttonUpClicked(self):
		items = self.selectedItems(self.listBox2)
		for item in items:
			index = self.listBox2.index(item)
			self.listBox2.takeItem(item)
			self.listBox2.insertItem(item, index - 1)
			self.listBox2.setCurrentItem(index - 1)
	
	def buttonDownClicked(self):
		items = self.selectedItems(self.listBox2)
		for item in items:
			index = self.listBox2.index(item)
			self.listBox2.takeItem(item)
			self.listBox2.insertItem(item, index + 1)
			self.listBox2.setCurrentItem(index + 1)

class AppearancePage(appearance_ui.Appearance):
	
	def __init__(self, *args):
		apply(appearance_ui.Appearance.__init__, (self,) + args)
		
		self.themeBox.insertStringList(dataBase.theme_qlist)
		theme_index = dataBase.theme_qlist.findIndex(dataBase.themeName(conf["theme"]))
		self.themeBox.setCurrentItem(theme_index)
		
		imgAdd = QImage(kde_iconPath + "16x16/actions/fileopen.png")
		imgAdd_pixmap = QPixmap(imgAdd)
		self.buttonAdd1.setPixmap(imgAdd_pixmap)
		
		self.iconBox.insertStringList(dataBase.icon_qlist)
		icon_index = dataBase.icon_qlist.findIndex(dataBase.iconName(conf["icon"]))
		self.iconBox.setCurrentItem(icon_index)
		
		self.buttonAdd2.setPixmap(imgAdd_pixmap)
		
		self.theme_img.setPixmap(dataBase.themePreviewQPixmap(dataBase.themeName(conf["theme"])))
		
		self.textFont = conf["font"]
		self.titleFont = conf["titleFont"]
		
		self.color = conf["color"]
		self.color = QColor(self.color[0], self.color[1], self.color[2])
# 		self.pixmap_color = QPixmap(80,14,-1)
# 		self.pixmap_color.fill(self.color)
# 		self.button_color.setPixmap(self.pixmap_color)
		self.textPreview.setText(self.textFont)
		self.textPreview.setFont(QFont(self.textFont))
		self.textPreview.setPaletteForegroundColor(self.color)
		
		self.textShadow = conf["shadow"]
		
		self.titleColor = conf["titleColor"]
		self.titleColor = QColor(self.titleColor[0], self.titleColor[1], self.titleColor[2])
# 		self.pixmap_titleColor = QPixmap(80,14,-1)
# 		self.pixmap_titleColor.fill(self.titleColor)
# 		self.button_titleColor.setPixmap(self.pixmap_titleColor)
		self.titlePreview.setText(self.titleFont)
		self.titlePreview.setFont(QFont(self.titleFont))
		self.titlePreview.setPaletteForegroundColor(self.titleColor)
		
		self.titleShadow = conf["titleShadow"]
		
		self.checkBox.setChecked(not(conf["animation"]))
		
		self.connect(self.themeBox, SIGNAL("activated(int)"), self.updateTheme)
		self.connect(self.buttonAdd1, SIGNAL("released()"), self.buttonAddThemeClicked)
		self.connect(self.buttonAdd2, SIGNAL("released()"), self.buttonAddIconClicked)
		self.connect(self.button_color, SIGNAL("released()"), self.button_colorClicked)
		self.connect(self.button_titleColor, SIGNAL("released()"), self.button_titleColorClicked)
	
	
	def buttonAddThemeClicked(self):
		path = str(QFileDialog.getExistingDirectory(conf["home"], self, "Theme", 'Theme path', True, True))
		if (path != ""):
			name = dataBase.addTheme(path)
			if (name != False):
				if (self.themeBox.count() != dataBase.theme_qlist.count()):
					self.themeBox.insertItem(name, -1)
				index = dataBase.theme_qlist.findIndex(name)
				self.themeBox.setCurrentItem(index)
				self.updateTheme(index)
			else:
				QMessageBox.information(self, "Invalid path", "This path doesn't seem to be a valid Aero AIO theme.")
	
	def buttonAddIconClicked(self):
		path = str(QFileDialog.getExistingDirectory(conf["home"], self, "Icons", 'Icon set path', True, True))
		if (path != ""):
			name = dataBase.addIcons(path)
			if (name != False):
				if (self.iconBox.count() != dataBase.icon_qlist.count()):
					self.iconBox.insertItem(name, -1)
				self.iconBox.setCurrentItem(dataBase.icon_qlist.findIndex(name))
			else:
				QMessageBox.information(self, "Error", "An error occured during icon set installation.")
	
	def button_colorClicked(self):
		aio = AIOFontDialog()
		aio_font = aio.getFont(None, self.textFont, self.color, self.textShadow)
		if (aio_font != None):
			self.textFont = aio_font.family
			self.color = aio_font.color
			self.textShadow = aio_font.shadow
			self.textPreview.setText(aio_font.family)
			self.textPreview.setFont(QFont(aio_font.family))
			self.textPreview.setPaletteForegroundColor(aio_font.color)
	# 		self.color = QColorDialog.getColor(self.color, self, "Choose text color")
	# 		self.pixmap_color.fill(self.color)
	# 		self.button_color.setPixmap(self.pixmap_color)
	
	def button_titleColorClicked(self):
		aio = AIOFontDialog()
		aio_font = aio.getFont(None, self.titleFont, self.titleColor, self.titleShadow)
		if (aio_font != None):
			self.titleFont = aio_font.family
			self.titleColor = aio_font.color
			self.titleShadow = aio_font.shadow
			self.titlePreview.setText(aio_font.family)
			self.titlePreview.setFont(QFont(aio_font.family))
			self.titlePreview.setPaletteForegroundColor(aio_font.color)
	# 		self.titleColor = QColorDialog.getColor(self.titleColor, self, "Choose titles' color")
	# 		self.pixmap_titleColor.fill(self.titleColor)
	# 		self.button_titleColor.setPixmap(self.pixmap_titleColor)
	
	def updateTheme(self, arg):
		name = str(dataBase.theme_qlist[arg])
		self.theme_img.setPixmap(dataBase.themePreviewQPixmap(name))
		dict = dataBase.themeDefault(name)
		self.color = dict["color"]
		self.textPreview.setPaletteForegroundColor(self.color)
		self.titleColor = dict["titleColor"]
		self.titlePreview.setPaletteForegroundColor(self.titleColor)
		icon_index = dataBase.icon_qlist.findIndex(dict["icon"])
		self.iconBox.setCurrentItem(icon_index)

class AutoswitchPage(auto_switching_ui.AutoSwitching):
	
	def __init__(self, *args):
		apply(auto_switching_ui.AutoSwitching.__init__, (self,) + args)
		
		self.autoswitch = conf["autoswitch"]
		length = len(self.autoswitch)
		
		if ( length > 2 ):
			self.checkBox.setChecked(True)
		else:
			self.checkBoxClicked()
		self.spinBox.setValue(self.autoswitch[0])
		
		imgClear = QImage(kde_iconPath + "22x22/actions/editdelete.png")
		imgClear_pixmap = QPixmap(imgClear)
		self.buttonClear.setPixmap(imgClear_pixmap)
		
		imgRight = QImage(kde_iconPath + "22x22/actions/forward.png")
		imgRight_pixmap = QPixmap(imgRight)
		self.buttonRight.setPixmap(imgRight_pixmap)
		
		imgLeft = QImage(kde_iconPath + "22x22/actions/back.png")
		imgLeft_pixmap = QPixmap(imgLeft)
		self.buttonLeft.setPixmap(imgLeft_pixmap)
		
		imgUp = QImage(kde_iconPath + "22x22/actions/up.png")
		imgUp_pixmap = QPixmap(imgUp)
		self.buttonUp.setPixmap(imgUp_pixmap)
		
		imgDown = QImage(kde_iconPath + "22x22/actions/down.png")
		imgDown_pixmap = QPixmap(imgDown)
		self.buttonDown.setPixmap(imgDown_pixmap)
		
		self.connect(self.checkBox, SIGNAL("clicked()"), self.checkBoxClicked)
		self.connect(self.buttonClear, SIGNAL("released()"), self.buttonClearClicked)
		self.connect(self.buttonRight, SIGNAL("released()"), self.buttonRightClicked)
		self.connect(self.buttonLeft, SIGNAL("released()"), self.buttonLeftClicked)
		self.connect(self.buttonUp, SIGNAL("released()"), self.buttonUpClicked)
		self.connect(self.buttonDown, SIGNAL("released()"), self.buttonDownClicked)
	
	def selectedItems(self, listBox):
		number = listBox.count()
		item_list = []
		for i in range(number):
			if listBox.isSelected(i):
				item_list.append(listBox.item(i))
		return item_list
	
	def checkBoxClicked(self):
		state = self.checkBox.isChecked()
		widgetList = [self.label1, self.label2, self.label3, self.listBox1, self.listBox2, self.spinBox, self.buttonClear, self.buttonUp, self.buttonDown, self.buttonRight, self.buttonLeft]
		for widget in widgetList:
			widget.setEnabled(state)
	
	def buttonClearClicked(self):
		self.listBox2.clear()
	
	def buttonUpClicked(self):
		items = self.selectedItems(self.listBox2)
		for item in items:
			index = self.listBox2.index(item)
			self.listBox2.takeItem(item)
			self.listBox2.insertItem(item, index - 1)
			self.listBox2.setCurrentItem(index - 1)
	
	def buttonDownClicked(self):
		items = self.selectedItems(self.listBox2)
		for item in items:
			index = self.listBox2.index(item)
			self.listBox2.takeItem(item)
			self.listBox2.insertItem(item, index + 1)
			self.listBox2.setCurrentItem(index + 1)
	
	def buttonRightClicked(self):
		items = self.selectedItems(self.listBox1)
		for item in items:
			new_item = PluginItem(self.listBox2, item.text(), item.plugin)
			self.listBox2.setSelected(new_item, False)
			self.listBox1.setSelected(item, False)
	
	def buttonLeftClicked(self):
		items = self.selectedItems(self.listBox2)
		for item in items:
			index = self.listBox2.index(item)
			self.listBox2.removeItem(index)

class PluginItem(QListBoxText):
	
	def __init__(self, parent, text, plugin):
		apply(QListBoxText.__init__, (self, parent, text))
		self.plugin = plugin

class MainDialog(config_dialog_ui.ConfigDialog):
	
	def __init__(self, *args):
		apply(config_dialog_ui.ConfigDialog.__init__, (self,) + args)
		
		self.setIcon(dataBase.iconQPixmap())
		self.pixmapLabel1.setPixmap(dataBase.logoQPixmap())
		
		self.general = self.listView.findItem("General", 0)
		self.general.setOpen(True)
		self.plugins = self.listView.findItem("Plugins", 0)
		self.plugins.setOpen(True)
		
		self._plugin = self.addPage(self.general, PluginsPage(), "Plugins Management")
		self._appearance = self.addPage(self.general, AppearancePage(), "Appearance")
		self._autoswitch = self.addPage(self.general, AutoswitchPage(), "Auto-Switching")
		
		self.new = []
		self.rem = []
		
		for gui in dataBase.gui_dict:
			f = compile(gui + '.Page(self,conf)', '<string>', 'eval')
			qwidget = eval(f)
			plug = gui[:-3] + "aio"
			name = conf["plugin2name_dict"][plug]
			self.addPage(self.plugins, qwidget, name)
			if not( plug in conf['page_list'] ):
				self.listView.findItem(name, 0).setVisible(False)
		
		for plugin in conf["page_list"]:
			PluginItem(self._plugin.listBox2, conf["plugin2name_dict"][plugin], plugin)
			PluginItem(self._autoswitch.listBox1, conf["plugin2name_dict"][plugin], plugin)
		
		for plugin in conf["plugin_dict"]:
			if not(plugin in conf["page_list"]):
				PluginItem(self._plugin.listBox1, conf["plugin2name_dict"][plugin], plugin)
		
		for plugin in conf["autoswitch"][1:]:
			PluginItem(self._autoswitch.listBox2, conf["plugin2name_dict"][plugin], plugin)
		
		self.connect(self.buttonOk, SIGNAL("released()"), self.OkClicked)
		self.connect(self.buttonApply, SIGNAL("released()"), self.ApplyClicked)
		self.connect(self.listView, SIGNAL("selectionChanged(QListViewItem*)"), self.itemClicked)
		self.connect(self._plugin.buttonAdd, SIGNAL("released()"), self.buttonAddClicked)
		self.connect(self._plugin.buttonRemove, SIGNAL("released()"), self.buttonRemoveClicked)
		self.connect(self._plugin.buttonClear, SIGNAL("released()"), self.buttonClearClicked)
		self.connect(self._plugin.buttonRight, SIGNAL("released()"), self.buttonRightClicked)
		self.connect(self._plugin.buttonLeft, SIGNAL("released()"), self.buttonLeftClicked)
		
	
	def addPage(self, parent, qwidget, name):
		num = self.widgetStack.addWidget(qwidget)
		ConfListViewItem(parent, name, num)
		return qwidget
	
	def itemClicked(self, item):
		if (item != self.general and item != self.plugins and item != None):
			self.widgetStack.raiseWidget(item.stackid)
	
	
	def buttonAddClicked(self):
		file_name = str(QFileDialog.getOpenFileName(conf["home"], "Plugins (*.aiz)", self, "Plugin_dir", 'Add/Install a plugin'))
		failed = None
		if (file_name.find(".aiz") != -1):
			plugin_file = zipfile.ZipFile(file_name, 'a')
			__list = plugin_file.namelist()
			file_list = []
			newdir_list = []
			for file in __list:
				match = re.search("(.*/)[^/]*", file)
				if (match != None):
					path = conf["home"] + ".aero_aio/" + match.group(1)
					if not(os.path.exists(path) or (path in newdir_list)):
						newdir_list.append(path)
				if (file[-1] != "/"):
					file_list.append(file)
			plugin = None
			count = 0
			for name in file_list:
				if (name.find("_aio.py") != -1 and name.find(".pyc") == -1 and name.find(".py~") == -1):
					count = count + 1
					plugin = name
			if (plugin != None and count == 1 and plugin.find("/") == -1):
				name = plugin_file.read(plugin)
				plugin = plugin[:-3]
				i1 = name.find("plugin_name")
				j1 = name.find("default_param")
				if ((i1 == -1) or (j1 == -1)):
					failed = "Invalid plugin file : " + plugin + ".py."
				else:
					for newdir in newdir_list:
						os.mkdir(newdir)
					i2 = name.find('\n', i1)
					j2 = name.find('}\n', j1)
					fex1 = compile(name[i1+7:i2], '<string>', 'exec')
					fex2 = compile(name[j1:j2+1], '<string>', 'exec')
					exec fex1
					exec fex2
					for f in file_list:
						string = plugin_file.read(f)
						try:
							a = open(conf["home"] + ".aero_aio/" + f, 'w')
							a.write(string)
							a.close()
						except:
							failed = "Can't copy files to local folder."
					try:
						a = open(conf["home"] + ".aero_aio/" + plugin + ".lst", 'w')
						file_list.append(plugin + ".pyc")
						a.write(str(file_list))
						a.close()
					except:
						failed = "Can't write file list to local folder."
			else:
				failed = "No plugin or several plugins found in archive."
			if (failed != None):
				QMessageBox.information(self, "Invalid plugin archive", "Invalid Aero AIO plugin archive :\n" + failed)
			else:
				conf['plugin2name_dict'][plugin] = name
				for param in default_param:
					conf[param] = default_param[param]
				gui = plugin[:-3] + "gui"
				if (plugin in conf["plugin_dict"]):
					if not(plugin in self.rem):
						self.rem.append(plugin)
					if gui in dataBase.gui_dict:
						item = self.listView.findItem(name,0)
						self.widgetStack.removeWidget(self.widgetStack.widget(item.stackid))
						fexec = compile("global " + gui + "\ndel " + gui, '<string>', 'exec')
						exec fexec
				else:
					PluginItem(self._plugin.listBox1, name, plugin)
				for f in file_list:
					if (f.find("scripts/" + plugin[:-3] + "gui.py") != -1 and f.find(".pyc") == -1 and f.find(".py~") == -1):
						dataBase.gui_dict[gui] = conf["home"] + ".aero_aio/scripts/" + gui
						__globalimport__({gui:dataBase.gui_dict[gui]})
						feval = compile(gui + '.Page(self,conf)', '<string>', 'eval')
						if (plugin in conf["plugin_dict"]):
							item.stackid = self.widgetStack.addWidget(eval(feval))
						else:
							self.addPage(self.plugins, eval(feval), name)
							self.listView.findItem(name, 0).setVisible(False)
							
				conf["plugin_dict"][plugin] = plugin
				if not(plugin in self.new):
					self.new.append(plugin)
				self.buttonCancel.setEnabled(False)
			plugin_file.close()
	
	def buttonRemoveClicked(self):
		items = self._plugin.selectedItems(self._plugin.listBox1)
		item_str = ""
		for item in items:
			item_str = item_str + str(item.text()) + ", "
		if (item_str != ""):
			confirm = QMessageBox.question(self, "Remove/Uninstall plugins", "Uninstall " + item_str[:-2] + "?", QMessageBox.Yes, QMessageBox.No, QMessageBox.NoButton)
			if (confirm == QMessageBox.Yes):
				for item in items:
					plugin = item.plugin
					if (os.path.exists(conf["home"] + ".aero_aio/" + plugin + ".lst")):
						f = open(conf["home"] + ".aero_aio/" + plugin + ".lst")
						file_list = eval(f.read())
						f.close()
						success = True
						for file in file_list:
							try:
								os.popen("rm -f " + conf["home"] + ".aero_aio/" + file)
							except:
								success = False
						try:
							os.popen("rm -f " + conf["home"] + ".aero_aio/" + plugin + ".lst")
						except:
							success = False
						if success:
							if not(plugin in self.rem):
								self.rem.append(plugin)
							text = item.text()
							self.plugins.takeItem(self.listView.findItem(text, 0))
							self._plugin.listBox1.removeItem(self._plugin.listBox1.index(self._plugin.listBox1.findItem(text)))
							self._autoswitch.listBox1.removeItem(self._autoswitch.listBox1.index(self._autoswitch.listBox1.findItem(text)))
							number = self._autoswitch.listBox2.count()
							num_list = []
							for i in range(number):
								itemf = self._autoswitch.listBox2.findItem(text)
								if (itemf != 0):
									self._autoswitch.listBox2.removeItem(self._autoswitch.listBox2.index(itemf))
							if (plugin in self.new):
								self.new.remove(plugin)
							del conf['plugin_dict'][plugin]
							self.buttonCancel.setEnabled(False)
						else:
							QMessageBox.information(self, "Remove/Uninstall plugins", "Can't erase files :\n you should remove them manually from " + conf["home"] + ".aero_aio/")
					else:
						QMessageBox.information(self, "Remove/Uninstall plugins", "This plugin can't be removed automatically, you should remove it manually.")
	
	def buttonClearClicked(self):
		number = self._plugin.listBox2.count()
		for i in range(number):
			item = self._plugin.listBox2.item(0)
			self._plugin.listBox2.takeItem(item)
			self._plugin.listBox1.insertItem(item, -1)
			self._plugin.listBox1.setSelected(item, False)
			gui_name = item.plugin[:-3] + "gui"
			if ( gui_name in dataBase.gui_dict ):
				self.listView.findItem(str(item.text()), 0).setVisible(False)
		self._autoswitch.listBox1.clear()
		self._autoswitch.listBox2.clear()
	
	def buttonRightClicked(self):
		items = self._plugin.selectedItems(self._plugin.listBox1)
		for item in items:
			self._plugin.listBox1.takeItem(item)
			self._plugin.listBox2.insertItem(item, -1)
			self._plugin.listBox2.setSelected(item, False)
			gui_name = item.plugin[:-3] + "gui"
			if ( gui_name in dataBase.gui_dict ):
				self.listView.findItem(str(item.text()), 0).setVisible(True)
			new_item = QListBoxText(item.text())
			new_item.plugin = item.plugin
			self._autoswitch.listBox1.insertItem(new_item, -1)
			self._autoswitch.listBox1.setSelected(new_item, False)
	
	def buttonLeftClicked(self):
		items = self._plugin.selectedItems(self._plugin.listBox2)
		for item in items:
			self._plugin.listBox2.takeItem(item)
			self._plugin.listBox1.insertItem(item, -1)
			self._plugin.listBox1.setSelected(item, False)
			gui_name = item.plugin[:-3] + "gui"
			text = item.text()
			if ( gui_name in dataBase.gui_dict ):
				self.listView.findItem(str(text),0).setVisible(False)
			self._autoswitch.listBox1.removeItem(self._autoswitch.listBox1.index(self._autoswitch.listBox1.findItem(text)))
			number = self._autoswitch.listBox2.count()
			num_list = []
			for i in range(number):
				itemf = self._autoswitch.listBox2.findItem(text)
				if (itemf != 0):
					self._autoswitch.listBox2.removeItem(self._autoswitch.listBox2.index(itemf))
	
	def writeConfig(self):
		color = [self._appearance.color.red(), self._appearance.color.green(), self._appearance.color.blue()]
		titleColor = [self._appearance.titleColor.red(), self._appearance.titleColor.green(), self._appearance.titleColor.blue()]
		shadow = self._appearance.textShadow
		titleShadow = self._appearance.titleShadow
		font = str(self._appearance.textFont)
		titleFont = str(self._appearance.titleFont)
		theme = dataBase.theme_dict[str(self._appearance.themeBox.currentText())]
		icon = dataBase.icon_dict[str(self._appearance.iconBox.currentText())]
		
		page_list = []
		number = self._plugin.listBox2.count()
		for i in range(number):
			page_list.append(self._plugin.listBox2.item(i).plugin)
		
		autoswitch = [self._autoswitch.spinBox.value()]
		if self._autoswitch.checkBox.isChecked():
			number = self._autoswitch.listBox2.count()
			for i in range(number):
				autoswitch.append(str(self._autoswitch.listBox2.item(i).plugin))
		
		animation = not(self._appearance.checkBox.isChecked())
		
		outTheme_list = []
		for name in dataBase.outTheme_list:
			outTheme_list.append(dataBase.theme_dict[name])
		
		outIcon_list = []
		for name in dataBase.outIcon_list:
			outIcon_list.append(dataBase.icon_dict[name])
		
		conf_returned = { "color" : color,
			"titleColor" : titleColor,
			"shadow" : shadow,
			"titleShadow" : titleShadow,
			"font" : font,
			"titleFont" : titleFont,
			"theme" : theme,
			"ext_themes" : outTheme_list,
			"icon" : icon,
			"ext_icons" : outIcon_list,
			"page_list" : page_list,
			"autoswitch" : autoswitch,
			"animation" : animation,
			"new" : self.new,
			"rem" : self.rem }
		
		for plugin in conf['plugin_dict']:
			item = self.listView.findItem(conf['plugin2name_dict'][plugin], 0)
			if (item != None):
				widget = self.widgetStack.widget(item.stackid)
				widget.buildParam()
				param = widget.param
				for key in param:
					conf_returned[key] = param[key]
				
				if (plugin in self.rem) and not(plugin in self.new):
					gui = plugin[:-3] + "gui"
					self.widgetStack.removeWidget(widget)
					del dataBase.gui_dict[gui]
		
		try:
			f = open(dataBase.conf_path, "w")
			f.write(str(time()) + "\n" + str(conf_returned))
		except:
			return False
		else:
			f.close()
			return True
	
	def OkClicked(self):
		conf_written = self.writeConfig()
		if conf_written:
			dataBase.close()
			print "CONF SAVED SUCCESSFULLY"
			self.accept()
		else:
			self.reject()
	
	def ApplyClicked(self):
		conf_written = self.writeConfig()
		if conf_written:
			self.buttonCancel.setEnabled(True)
			self.new = []
			self.rem = []
		else:
			self.reject()

if ( __name__ == '__main__' ):
	app = QApplication(sys.argv)
	win = MainDialog()
	app.setMainWidget(win)
	win.show()
# 	app.connect(win.buttonCancel, SIGNAL("released()"), app, SLOT("quit()"))
	app.connect(app, SIGNAL("lastWindowClosed()"), app, SLOT("quit()"))
	
	''' Enter main loop '''
	app.exec_loop()